classdef Job < handle
    properties (SetAccess = private)
        is_submitted
        is_crashed
        use_cluster
        use_GE
        cluster_id
        name
        stamp
        commands
        dependencies
    end
    properties (Dependent)
        is_finished
    end
    
    methods
        function obj = Job(use_cluster, use_GE, name, stamp, commands, dependencies)
            obj.is_submitted = false;
            obj.is_crashed = false;
            obj.use_cluster = use_cluster;
            obj.use_GE = use_GE;
            obj.name = name;
            obj.stamp = stamp;
            obj.dependencies = dependencies;
            
            % load libraries, set variables, change directory to scratch,
            % attach given commands
            if use_cluster
                fid = fopen(FilesDirs.prep_job_cluster_filename);
                if fid == -1 
                    error(['Unable to open: ' FilesDirs.prep_job_cluster_filename]) 
                end
            else
                fid = fopen(FilesDirs.prep_job_local_filename);
                if fid == -1 
                    error(['Unable to open: ' FilesDirs.prep_local_job_filename]) 
                end
            end
            obj.commands = {};
            while 1
                tline = fgetl(fid);
                if tline == -1, break, end
                obj.commands = [obj.commands tline];
            end
            obj.commands = [obj.commands ['cd ' FilesDirs.tmp_dir()]];
            obj.commands = [obj.commands commands];
            fclose(fid);
        end
        
        function is_finished = get.is_finished(obj)
            if (obj.use_cluster)
                fid = fopen(FilesDirs.jpHMM_res_filename(obj.stamp));
                %disp(FilesDirs.jpHMM_res_filename(obj.stamp))
                is_finished = (fid ~= -1);
                if is_finished
                    fclose(fid);    
                end
            else
                is_finished = true;
            end
        end
        
        function obj = submit(obj)
            global manual_cluster;
            obj.is_submitted = true;
            if (obj.use_cluster)
                % generate script
                script_filename = FilesDirs.script_filename(obj.name, obj.stamp);
                fid = fopen(script_filename, 'w');
                if fid == -1 
                    error(['Unable to open: ' script_filename]) 
                end   
                for i=1:length(obj.commands)
                    fprintf(fid, '%s\n', obj.commands{i});
                end
                fclose(fid);
                fileattrib(script_filename,'+x','u');
                
                if (obj.use_GE)
                    % process dependencies
                    dep_str = '';
                    for i=1:length(obj.dependencies)
                        if (i > 1) 
                            dep_str = [dep_str ','];
                        end
                        dep_str = [dep_str num2str(obj.dependencies(i))];
                    end

                    % submit script with qsub
                    submit_cmd = ['ssh fe ''. /opt/sge/default/common/settings.sh; qsub -N ' obj.name ' -o ' FilesDirs.cluster_output_dir ' -e ' FilesDirs.cluster_error_dir ' '];
                    if (~isempty(obj.dependencies))
                        submit_cmd = [submit_cmd '-hold_jid ' dep_str ' '];
                    end
                    submit_cmd = [submit_cmd script_filename ' '' '];
                    disp('Not checking whether GE is full')
%                     while (GridEngine.num_running_jobs() > 90)
%                         pause(5);
%                     end
                    disp(submit_cmd);
                    [st, rs] = system(submit_cmd);                
                    expr = 'Your job (\d{7}) .* has been submitted';
                    [tokens] = regexp(rs, expr, 'tokens');
                    [s1 s2] = size(tokens);          
                    if (s1 == 1) && (s2 == 1)
                        id = str2double(tokens{1}{1});
                        obj.cluster_id = id;
                        disp(['Job ' num2str(id) ' has been submitted'])
                    else
                        obj.is_crashed = true;
                        disp('A job has crashed')
                    end
                else
                    if (~isempty(obj.dependencies))
                        disp('Used dependencies for non-GE job, ignoring');
                    end  
                    node_number = manual_cluster.register_job(FilesDirs.ep_filename(obj.stamp));
                    submit_cmd = ['ssh node' node_number ' "' script_filename ' 2>&1>/dev/null" & ']                          
                    system(submit_cmd);
                end
            else
                exec_cmd = '';
                for i=1:length(obj.commands)
                    exec_cmd = [exec_cmd obj.commands{i} '; '];
                end   
                system(exec_cmd);
            end
        end
    end
end
