classdef MachLearn
    methods (Static)
        function job = create_job(stamp)
            global consts;           
            cmd = [FilesDirs.ml_ext_dir '/machLearnExt ' stamp];     
            job = Job(consts.use_cluster, false, 'cep_ml', stamp, cmd, []);
        end
        
        % calculate emission probs using machine learning approach
        function calc_em_probs(test_seq, mach_learn_para, al, stamp)
            global consts;            
            disp('calc_em_probs');
            subs = sort([mach_learn_para.small_subs mach_learn_para.large_subs]);
            
            is_first = true;
            begin_sectors = [];
            end_sectors = [];
            curr_sector_begin = test_seq.begin_pos;
            pos = test_seq.begin_pos + 1;
            while is_first || (pos < test_seq.end_pos)
                while (pos < al.seq_len) && (al.num_residuals(curr_sector_begin, pos) < mach_learn_para.res_per_sector)
                    pos = pos + 1;
                end
                begin_sectors = [begin_sectors curr_sector_begin];
                end_sectors = [end_sectors pos];
                is_first = false;
                curr_sector_begin = pos + 1;
            end
            
            W = [];
            for i=1:length(begin_sectors)
%                 disp(['Sector ' num2str(i) ' of ' num2str(length(begin_sectors))])
                curr_W = MachLearn.calc_weights(test_seq, begin_sectors(i), end_sectors(i), mach_learn_para, al);
                W = vertcat(W, curr_W);
            end
            W = W(1:test_seq.len * consts.alphabet.num_nucl,:);
            
            % output weights
            if consts.output_ml_weights
                fid = FilesDirs.sec_fopen(FilesDirs.ml_weights_filename(stamp), 'w');
                [num_rows num_cols] = size(W);
                for i=1:num_rows
                    c = consts.alphabet.idx2char(mod(i - 1, consts.alphabet.num_nucl) + 1);
                    pos = num_rows / consts.alphabet.num_nucl - ceil(i / consts.alphabet.num_nucl);
                    fprintf(fid, '%4d %c  ', pos, c);
                    for j=1:num_cols                        
                        fprintf(fid, '% 6e  ', W(i, j));
                    end
                    fprintf(fid, '\n');
                end
                fclose(fid);
            end
            
            % calculate emission probs
            em_probs = zeros(length(subs), test_seq.len, consts.alphabet.num_nucl);
            gamma = mach_learn_para.gamma;            
            for i=1:length(subs)
                for j=1:test_seq.len
                    v = zeros(1, consts.alphabet.num_nucl);
                    for k=1:consts.alphabet.num_nucl
                        v(k) = exp(gamma * W((j - 1) * consts.alphabet.num_nucl + k, i));
                    end
                    for k=1:consts.alphabet.num_nucl
                        em_probs(i, j, k) = v(k) / sum(v);
                    end
                end
            end
            
            % save emission probs
            fid = fopen(FilesDirs.ep_filename(stamp), 'w');
            if fid == -1 
                error(['Unable to open: ' FilesDirs.ep_filename(stamp)]) 
            end

            for i=1:length(subs)
                for j=1:test_seq.len
                    fprintf(fid, '%i %s', j-1, subs{i});
                    for k=1:consts.alphabet.num_nucl
                        fprintf(fid, ' %d', em_probs(i, j, k));
                    end
                    fprintf(fid, '\n');
                end
            end            
            
            fclose(fid);
        end        
    end
    
    % calculate weights
    methods (Static, Access = private)
        function W = calc_weights(test_seq, begin_sector, end_sector, mach_learn_para, al)
            global consts;
            subs = sort([mach_learn_para.small_subs mach_learn_para.large_subs]);
            len_sector = end_sector - begin_sector + 1;
            
            T = [];
            X = [];
            M = [];           
            for i=1:length(subs)               
                sub = char(subs{i});
%                 disp(['calculating weights for Sub ' sub]);
                no_curr_used_seqs = al.no_seqs_of_sub(sub);
                no_curr_used_seqs = setdiff(no_curr_used_seqs, test_seq.no_used_seqs);
                num_seqs = length(no_curr_used_seqs);
                % T
                curr_row = zeros(1, length(subs));
                curr_row(i) = 1;
                for j=1:length(no_curr_used_seqs)
                    T = [T; curr_row];
                end
                % X
                curr_X = [];
                for j=1:length(no_curr_used_seqs)
                    curr_row = zeros(1, consts.alphabet.num_nucl * len_sector);
                    for k=1:len_sector
                        idx = consts.alphabet.char2idx(al.seqs(no_curr_used_seqs(j)).genome(begin_sector + k - 1));
                        curr_row((k - 1) * consts.alphabet.num_nucl + idx) = 1;
                    end
                    curr_X = [curr_X; curr_row];
                end
                X{i} = curr_X;
                % M
                curr_M = X{i}.' * ones(length(no_curr_used_seqs), 1) / num_seqs;
                M = [M curr_M];
            end
           
            lambda = mach_learn_para.lambda;
            C = zeros(len_sector * consts.alphabet.num_nucl);
            for i=1:length(subs)
                C = C + X{i}.' * X{i} / num_seqs;
            end
            L = lambda * eye(len_sector * consts.alphabet.num_nucl);
            W = (C + L) \ M;
        end
    end    
end