classdef Subs
    properties (SetAccess = private, GetAccess = private)
        seqs
    end
    properties (SetAccess = private)
        names
        num_seqs
    end
    properties (Dependent)
        num
    end
    
    methods
        function s = Subs(names)
            s.names = names;
            s.seqs = containers.Map(names, cell(1, length(names)));
            values = zeros(1, length(names));
            s.num_seqs = containers.Map(names, mat2cell(values, 1, ones(1, length(names))));
        end
        
        function num = get.num(obj)
            num = length(obj.names);
        end
        
        function obj = add_seq(obj, seq)
            obj.seqs(seq.sub) = [obj.seqs(seq.sub) seq];  
            obj.num_seqs(seq.sub) = obj.num_seqs(seq.sub) + 1;
        end
        
        function [seqs] = get_seqs(obj, sub_str)
            seqs = obj.seqs(sub_str);
        end
        
        function idx = sub2idx(obj, sub_str)
            idx = find(strcmp(sub_str, obj.names));
        end
        
        function idx = sub2c_idx(obj, sub_str)
            idx = sub2idx(obj, sub_str) -  1;
        end
  
        function sub = c_idx2sub(obj, idx)
            sub = obj.names{idx + 1};
        end
        
        function b = is_valid(obj, sub_str)
            b = any(strcmp(sub_str, obj.names));
        end
        
        % conservation propensity
        function prop = cons_prop(obj, sub_str)
            seqs = obj.seqs(sub_str);
            pos_cons_prop = [];
            for i = 1:seqs(1).len
                i
                freqs = containers.Map({'A', 'C', 'G', 'T'}, {0, 0, 0, 0});
                for j = 1:length(seqs)
                    curr_nucl = seqs(j).genome(i);
                    if (freqs.isKey(curr_nucl))
                        freqs(curr_nucl) = freqs(curr_nucl) + 1;
                    end
                end
                curr_max_freq = max(cell2mat(values(freqs)));
                if (curr_max_freq > 0)
                    pos_cons_prop = [pos_cons_prop curr_max_freq / sum(cell2mat(values(freqs)))];
                end
            end
            prop = sum(pos_cons_prop) / length(pos_cons_prop);
        end
    end
end