clear all;
clear classes;
close all;
clc; 

tic;

javaaddpath('/home/ingo/Software/mysql-connector-java-3.1.14-bin.jar');
javaaddpath(pwd);
save_to_db = 1;
use_cluster = 0;
use_scratch = 0;

if (~save_to_db)
    disp('No SQL DB used');
end

s=RandStream('mt19937ar','Seed', 0);
RandStream.setDefaultStream(s)

global consts;
global all_subs;
global small_subs;
global large_subs;
global scenario;
global manual_cluster;

% scenario = 'simple';
% scenario = 'med';
% scenario = 'cpx';
scenario = 'compl';
% scenario = 'with_crfs';
load_vars = 1;
save_al_to_db = 0;
switch scenario
    case 'simple'
        al_id = 2;
    case 'med'
        al_id = 3;
    case 'cpx'
        al_id = 4;
    case 'compl'
        al_id = 1;
    case 'with_crfs'
        al_id = 1;
    otherwise
        error 'invalid scenario'
end

strategy = 'orig';
% strategy = 'prob';
% strategy = 'ml';
gen_repr_freqs = 0;

if (use_cluster)
    use_scratch = 1;
end
if (~strcmp(scenario, 'compl') && ~strcmp(scenario, 'with_crfs'))
    save_al_to_sql = 0;
end

consts = Constants(strategy, use_cluster, use_scratch, save_to_db);
if (use_cluster)
    manual_cluster = ManualCluster();
    disp(manual_cluster);
end

switch scenario
    case 'simple'
        al_matlab_filename = [FilesDirs.matlab_var_dir '/simple_test.mat'];
    case 'med'
        al_matlab_filename = [FilesDirs.matlab_var_dir '/med_test.mat'];
    case 'cpx'
        al_matlab_filename = [FilesDirs.matlab_var_dir '/cpx_test.mat'];    
    case 'compl'
        al_matlab_filename = [FilesDirs.matlab_var_dir '/compl_align.mat'];
    case 'with_crfs'
        al_matlab_filename = [FilesDirs.matlab_var_dir '/compl_align.mat'];
    otherwise
        error 'invalid scenario'
end

if (load_vars)
    load(al_matlab_filename);
    disp('variables loaded')
else
    switch scenario
        case 'simple'
            all_subs = Subs(sort({'G' 'H' 'J'}));
            small_subs = {'H' 'J'};
            large_subs = {'G'};
            al_fasta_filename = '/home/ingo/workspace/small_subtypes/input/simple_test.fa';
        case 'med'
            all_subs = Subs(sort({'G' 'H' 'J'}));
            small_subs = {'H' 'J'};
            large_subs = {'G'};
            al_fasta_filename = '/home/ingo/workspace/small_subtypes/input/med_test.fa';
        case 'cpx'
            all_subs = Subs(sort({'F' 'G' 'K'}));
            small_subs = {'K'};
            large_subs = {'F' 'G'};
            al_fasta_filename = '/home/ingo/workspace/small_subtypes/input/cpx_test.fa';
        case 'compl'
            all_subs = Subs({'A' 'B' 'C' 'D' 'F' 'G' 'H' 'J' 'K'});
            small_subs = {'H' 'J' 'K'};
            large_subs = {'A' 'B' 'C' 'D' 'F' 'G'};
            al_fasta_filename = '/home/ingo/workspace/small_subtypes/input/compl_align.fa';
        case 'with_crfs'
            all_subs = Subs({'A' 'B' 'C' 'D' 'F' 'G' 'H' 'J' 'K' '04' '06' '11' '13' '18'});
            small_subs = {'H' 'J' 'K'};
            large_subs = {'A' 'B' 'C' 'D' 'F' 'G'};
            al_fasta_filename = '/home/ingo/workspace/small_subtypes/input/compl_align.fa';
        otherwise
            error 'invalid scenario'
    end

    al = Alignment(al_fasta_filename, scenario);
    save(al_matlab_filename, 'al', 'all_subs', 'small_subs', 'large_subs');
end

if (save_al_to_db)
    al.save_to_db(al_id);
end
al.print_calc_etp_al(scenario);

if gen_repr_freqs
    al.gen_repr_freqs();
end

switch scenario
    case 'simple'
        test_seqs = TestSeqs({'H'}, {'G'}, 1, 0, 1, 30, 100, 10, 10, 30, 50, 10, al);
    case 'med'
%         test_seqs = TestSeqs({'H'}, {'G'}, 1, 1, 0, 2000, 3000, 50, 50, 2000, 300, al);
    case 'cpx'
%         test_seqs = TestSeqs({'K'}, {'F'}, 1, 1, 0, 2000, 3000, 10, 10, 2000, 2000, al);
    case 'compl'
%         test_seqs = TestSeqs({'H' 'J' 'K'}, {'A' 'B' 'C' 'D' 'F' 'G'}, 1, 10, 0, 2000, 7000, 250, 250, 2000, 3000, 2000, al);
        test_seqs = TestSeqs({}, {'A' 'B' 'C' 'D' 'F' 'G' 'H' 'J' 'K'}, 1, 0, 20, 2000, 7000, 250, 250, 2000, 7000, 250, al);
    case 'with_crfs'
        test_seqs = TestSeqs({}, {'04'}, 1, 0, 1, 2000, 7000, 250, 250, 1000, 1000, 9000, al);
%         test_seqs = TestSeqs({}, {'06'}, 1, 0, 1, 2000, 7000, 250, 250, 1000, 1000, 8500, al);
%         test_seqs = TestSeqs({}, {'11'}, 1, 0, 10, 2000, 7000, 250, 250, 1, 1, 100000, al);
%         test_seqs = TestSeqs({}, {'13'}, 1, 0, 1, 2000, 7000, 250, 250, 1, 1, 100000, al);
%         test_seqs = TestSeqs({}, {'18'}, 1, 0, 1, 2000, 7000, 250, 250, 1500, 1500, 8500, al);
    otherwise
        error 'invalid scenario'
end

switch strategy
    case 'orig'        
        jp = logspace(-20, -20, 1);
        mu = 1;
        for i = 1:length(jp)
            for j = 1:length(mu)
                model_para = OrigPara(jp(i), mu(j));
                res = model_para.score(test_seqs, al);
                disp('para:')
                disp(model_para)
                disp(['score: ' num2str(res.score)])
                disp(['score (sens): ' num2str(res.sens)])
                disp(['score (spec): ' num2str(res.spec) char(13)])
                for j=1:length(res.orig_segms)
                    disp(['sequence ' num2str(j) ':'])
                    disp(['score: (' num2str(res.seq_pos_scores(j)) ', ' num2str(res.seq_segm_scores(j)) ')'])
                    disp(['orig: ' Segmentation.segm2str(res.orig_segms(j), 1) ' ' Segmentation.segm2str(res.orig_segms(j), 0)])
                    disp(['est:  ' Segmentation.segm2str(res.est_segms(j), 1)  ' ' Segmentation.segm2str(res.est_segms(j), 0) char(13)])
                end    
            end
        end        
    case 'prob'    
        jp = logspace(-20, -20, 1);
        for i = 1:length(jp)
            model_para = BasicPara(jp(i));
            res = model_para.score(test_seqs, al);
            disp('para:')
            disp(model_para)
            disp(['score: ' num2str(res.score)])
            disp(['score (sens): ' num2str(res.sens)])
            disp(['score (spec): ' num2str(res.spec) char(13)])
            for j=1:length(res.orig_segms)
                disp(['sequence ' num2str(j) ':'])
                disp(['score: (' num2str(res.seq_pos_scores(j)) ', ' num2str(res.seq_segm_scores(j)) ')'])
                disp(['orig: ' Segmentation.segm2str(res.orig_segms(j), 1) ' ' Segmentation.segm2str(res.orig_segms(j), 0)])
                disp(['est:  ' Segmentation.segm2str(res.est_segms(j), 1)  ' ' Segmentation.segm2str(res.est_segms(j), 0) char(13)])
            end
        end        
    case 'ml'
        jp = logspace(-6, -6, 1);
        gamma_div_lambda = [3];
        for i = 1:length(jp)
            for j = 1:length(gamma_div_lambda)
                model_para = MachLearnPara(small_subs, large_subs, jp(i), 1000, gamma_div_lambda(j), 100);
                res = model_para.score(test_seqs, al);
                disp('para:')
                disp(model_para)
                disp(['score: ' num2str(res.score)])
                disp(['score (sens): ' num2str(res.sens)])
                disp(['score (spec): ' num2str(res.spec) char(13)])
                for j=1:length(res.orig_segms)
                    disp(['sequence ' num2str(j) ':'])
                    disp(['score: (' num2str(res.seq_pos_scores(j)) ', ' num2str(res.seq_segm_scores(j)) ')'])
                    disp(['orig: ' Segmentation.segm2str(res.orig_segms(j), 1) ' ' Segmentation.segm2str(res.orig_segms(j), 0)])
                    disp(['est:  ' Segmentation.segm2str(res.est_segms(j), 1)  ' ' Segmentation.segm2str(res.est_segms(j), 0) char(13)])
                end
            end
        end
    otherwise
        error 'Invalid method'   
end

if (~consts.use_scratch)
    disp('Scratch not used')
end

toc


